@@ -14,10 +14,12 @@ use IO::Socket::SSL;
 use JSON;
 use Time::HiRes qw(time);
 use URI::Escape;
+use UUID::Tiny ':std';
+
 
 use vars qw($VERSION);
 
-$VERSION = '1.10';
+$VERSION = '1.11';
 
 our $DEBUG = 0;
 
@@ -261,13 +263,10 @@ sub _unroll_ids {
 
     my( @items ) = ( map { $store->fetch($_) } @$ids );
 
-#    print STDERR Data::Dumper->Dump(["INITIAL IDS -->" . join(",",@$ids),"IDS FROM THOSE -->" . join(",",map { $store->_get_id($_) } @items ),"SEEN --> " .join(",",keys %$seen),"UNROLL"]);
-    
     my @outids;
     for my $item( @items ) {
         my $iid = $store->_get_id($item);
         my $r = ref( $item );
-#        print STDERR Data::Dumper->Dump(["  CHECKING ($iid) ($r)"]);
         $seen->{$iid}++;
         if( $r eq 'ARRAY' ) {
             push @outids, grep { ! $seen->{$_}++ } map { $store->_get_id($_)  } grep { ref($_) } @$item;
@@ -277,17 +276,13 @@ sub _unroll_ids {
         }
         else {
             my $data = $item->{DATA};
-#            print STDERR Data::Dumper->Dump([$data, "DATA for $item->{ID}"]);
             push @outids, map { $data->{$_} } grep { /^[^_]/ && $data->{$_} != /^v/ && ! $seen->{$data->{$_}}++ } keys %$data;
         }
     }
-#    print STDERR Data::Dumper->Dump(["SEEN --> " .join(",",keys %$seen),"OUTY --> " .join(",",@outids),"SEENY?"]);
-    
+
     _unroll_ids( $store, \@outids, $seen ) if @outids;
 
 
-#    print STDERR Data::Dumper->Dump(["SEEN --> " .join(",",keys %$seen),"SEEN NOWUNROLL!"]);
-    
     [ keys %$seen ];
 } #_unroll_ids
 
@@ -397,7 +392,7 @@ sub _process_request {
         elsif( $@ ) {
             _log( "ERROR : $@>" );
             $out_json = to_json( {
-                err => 'ERROR',
+                err => $@,
                                  } );
         }
         my @headers = (
@@ -424,33 +419,25 @@ sub invoke_payload {
     _log( "jsony $_[0]" );
 
     my $req_data = from_json( shift );
-
+    my( $file_uploads ) = shift;
     my( $obj_id, $token, $action, $params ) = @$req_data{ 'i', 't', 'a', 'pl' };
-#    print STDERR Data::Dumper->Dump(["'$action' on '$obj_id' with token [$token]) and params --> : ".join( ",", @$params )]);
     my $server_root = $self->{STORE}->fetch_server_root;
-    
+
     my $server_root_id = $server_root->{ID};
     my $session = $token && $token ne '_' ? $server_root->_fetch_session( $token ) : undef;
 
-#    print STDERR Data::Dumper->Dump(["TOKEN [$token] --> Session ($session)"]);
-
-    if( $session ) {
-#        print STDERR Data::Dumper->Dump([$session->{DATA},"SESSDA"]);
-    }
-    
     my $ids2times;
     if( $session ) {
         $ids2times = $session->get__has_ids2times;
-#        print STDERR Data::Dumper->Dump(["SESSION ($session) HAS --> " . join(',',sort {$a <=>$b }keys %$ids2times)]);
     }
-    
+
     unless( $obj_id eq '_' || 
             $obj_id eq $server_root_id ||
             ( $ids2times->{$obj_id} ) ) {
         # tried to do an action on an object it wasn't handed. do a 404
         die( "client with token [$token] and session ($session) tried to invoke on obj id '$obj_id' which it does not have" );
     }
-    if( substr( $action, 0, 1 ) eq '_' ) {
+    if( substr( $action, 0, 1 ) eq '_' || $action =~ /^[gs]et$/ ) {
         die( "Private method called" );
     }
 
@@ -464,7 +451,7 @@ sub invoke_payload {
     # are always a list, but they may contain other containers that are not
     # yote objects. So, transform the incomming parameter list and check all
     # yote objects inside for may. Use a recursive helper function for this.
-    my $in_params = $store->__transform_params( $params, $session );
+    my $in_params = $store->__transform_params( $params, $session, $file_uploads );
 
     #
     # This obj is the object that the method call is on
@@ -482,9 +469,7 @@ sub invoke_payload {
     }
 
     # <<------------- the actual method call --------------->>
-#    print STDERR "<".ref($obj)."><$action>\n";
     my(@res) = ($obj->$action( @$in_params ));
-#    print STDERR Data::Dumper->Dump([\@res,"MYREZ"]);
     # this is included in what is  returned to the client 
     my $out_res = $store->_xform_in( \@res, 'allow datastructures' );
 
@@ -492,8 +477,6 @@ sub invoke_payload {
     # the ids that were referenced explicitly in the
     # method call.
     my @out_ids = _find_ids_in_data( $out_res );
-#    print STDERR Data::Dumper->Dump(["RESULTS [$token] ($session) --> " . join(',',@res),"RESSU"]);
-#    print STDERR Data::Dumper->Dump(["OUT IDS [$token] ($session) --> ". join(",",@out_ids)]);
 
     #
     # Based on the return value of the method call,
@@ -501,7 +484,6 @@ sub invoke_payload {
     #   We will check to see if these need updates
     #
     my @should_have = ( @{ _unroll_ids( $store, \@out_ids ) } );
-#    print STDERR Data::Dumper->Dump(["SHOULDS [$token] ($session) --> " . join(",",@should_have)]);
 
     my( @updates, %methods );
 
@@ -556,7 +538,7 @@ sub invoke_payload {
             # check if this needs an update
             my( $client_s, $client_ms )  = @{ $ids2times->{$should_have_id} || [] };
             my( $server_s, $server_ms )  = $store->_last_updated( $should_have_id );
-#            print STDERR Data::Dumper->Dump(["Check ($should_have_id) --> c($client_s,$client_ms) <=> s($server_s,$server_ms) ". ( ( $client_s == 0 || $server_s > $client_s || ($server_s == $client_s && $server_ms > $client_ms )) ? " INCLUDE " : " DONT BOTHER ")]);
+
             if( $client_s == 0 || $server_s > $client_s || ($server_s == $client_s && $server_ms > $client_ms )) {
                 my $should_have_obj = $store->fetch( $should_have_id );
                 my $ref = ref( $should_have_obj );
@@ -577,18 +559,15 @@ sub invoke_payload {
                 };
                 push @updates, $update;
                 $ids2times->{$should_have_id} = [Time::HiRes::gettimeofday];
-#                print STDERR Data::Dumper->Dump(["ADDING '$should_have_id' to ids2times"]);
             }
         } #each object the client should have
-#        print STDERR Data::Dumper->Dump(["TOKEN [$token] SESSION ($session) NOW HAS {".$self->{STORE}->_get_id($ids2times)."} --> " . join(',',sort {$a <=>$b }keys %$ids2times)]);
     }
-    
+
     my $out_json = to_json( { result  => $out_res,
                               updates => \@updates,
                               methods => \%methods,
                             } );
     delete $obj->{SESSION};
-#    print STDERR Data::Dumper->Dump([$out_json,"OUTTTY"]);
     $self->{STORE}->stow_all;
 
     return $out_json;
@@ -663,23 +642,55 @@ sub __transform_params {
     # Throws execptions if the parametsr are not allowed, or if a reference that is not a hash or array
     # is encountered.
     #
-    my( $self, $param, $session ) = @_;
+    my( $self, $param, $session, $files ) = @_;
 
     if( ref( $param ) eq 'HASH' ) {
-        return { map { $_ => $self->__transform_params($param->{$_}, $session) } keys %$param };
+        return { map { $_ => $self->__transform_params($param->{$_}, $session, $files) } keys %$param };
     } 
     elsif( ref( $param ) eq 'ARRAY' ) {
-        return [ map { $self->__transform_params($_, $session) } @$param ];
+        return [ map { $self->__transform_params($_, $session, $files) } @$param ];
     } elsif( ref( $param ) ) {
         die "Transforming Params: got weird ref '" . ref( $param ) . "'";
     }
-    if( index( $param, 'v' ) != 0 && !$session->get__has_ids2times({})->{$param} ) {
+    if( ( index( $param, 'v' ) != 0 && index($param, 'f' ) != 0 ) && !$session->get__has_ids2times({})->{$param} ) {
         # obj id given, but the client should not have that id
         die "Client requested obj with id '$param' which it should not have.";
     }
-    return $self->_xform_out( $param );
+    return $self->_xform_out( $param, $files );
 } #__transform_params
 
+sub _xform_out {
+    my( $self, $val, $files ) = @_;
+    return undef unless defined( $val );
+    if( index($val,'f') == 0 ) {
+        # convert to file object
+        if( $val =~ /^f(\d+)_(\d+)$/ ) {
+            my( $offset_start, $offset_end ) = ( $1, $2 );
+            for( my $i=$offset_start; $i < $offset_end; $i++ ) {
+                my $file = $files->[$i];
+                my( $orig_filename ) = ( $file =~ /([^\/]*)$/ );
+                my( $extension ) = ( $orig_filename =~ /\.([^.\/]+)$/ );
+
+                # TODO - cleanup, maybe use File::Temp or something
+                my $newname = "/tmp/".create_uuid_as_string();
+                open (FILE, ">$newname");
+                while (read ($file, my $Buffer, 1024)) {
+                    print FILE $Buffer;
+                }
+                close FILE;
+
+                # create yote object here that wraps the file name
+                return $self->newobj( {
+                    file_path => $newname,
+                    file_extension => $extension,
+                    file_name => $orig_filename,
+                                      } );
+            }
+        }
+    }
+    return $self->SUPER::_xform_out( $val );
+} #_xform_out
+
 
 #
 # Unlike the superclass version of this, this provides an arguemnt to
@@ -706,21 +717,6 @@ sub _xform_in {
     return defined $val ? "v$val" : undef;
 } #_xform_in
 
-
-sub _find_ids_referenced {
-    my( $self, @ids ) = @_;
-    my( @refd );
-    for my $obj (map { $self->_xform_out( $_ ) } @ids ) {
-        if( ref $obj eq 'ARRAY' ) {
-            push @refd, grep { index($_,'v')!=0 } map { $self->_xform_in( $_ ) } @$obj;
-        } elsif( ref $obj eq 'HASH' ) {
-            push @refd, grep { index($_,'v')!=0 } map { $self->_xform_in( $_ ) } values %$obj;
-        } else {
-            push @refd, grep { index($_,'v')!=0 } map { $self->_xform_in( $_ ) } map { $obj->{DATA}{$_} } grep { index($_,'_') != 0 } keys %{$obj->{DATA}};
-        }
-    }
-}
-
 sub newobj {
     my( $self, $data, $class ) = @_;
     $class ||= 'Yote::ServerObj';
@@ -856,16 +852,13 @@ sub _fetch_session {
     
     $self->{STORE}->lock( 'token_mutex' );
     my $slots = $self->get__token_timeslots();
-#    print STDERR Data::Dumper->Dump(["FETCH SESSION GOT SLOTS (".$self->{STORE}->_get_id($slots).") for token [$token]"]);
     for( my $i=0; $i<@$slots; $i++ ) {
-#        print STDERR Data::Dumper->Dump(["   SLOT (".$self->{STORE}->_get_id($slots->[$i]).") has <".join(",",keys %{$slots->[$i]{$token}}).">"]);
         if( my $session = $slots->[$i]{$token} ) {
             #if( $i < $#$slots ) {
             if( $i > 0 ) {
                 # make sure this is in the most current 'boat'
                 $slots->[0]{ $token } = $session;
             }
-#            print STDERR Data::Dumper->Dump(["FOUND SESSION ($session)"]);
             $self->{STORE}->unlock( 'token_mutex' );
             return $session;
         }
@@ -911,7 +904,7 @@ sub _create_session {
     for( my $i=0; $i<@$slot_data; $i++ ) {
         return $self->_create_session( $tries++ ) if $slots->[ $i ]{ $token };
     }
-#    print STDERR Data::Dumper->Dump(["NEW TOKEN {{{$token}}"]);
+
     #
     # See if the most recent time slot is current. If it is behind, create a new current slot
     # create a new most recent boat.
